/*
** Kurpiel, A. and T. Roncalli [1998], Hopscotch methods for two-state
** financial models, FERC working paper
**
*/


new;
library pde2D,pgraph;
PDE2Dset;

declare matrix e;

rndseed 123;

sigma = 0.20;
r = 0.04;
mu = 0.08;
I = 1;
eta = 0.1;
Vbar = 1.5;

proc aProc(t,x,y);
  retp( 0.5*(sigma^2).* (x^e) .* ones(rows(x),cols(y)) );
endp;

proc bProc(t,x,y);
  retp( 0 .* ones(rows(x),cols(y)) );
endp;

proc cProc(t,x,y);
  retp( 0 .* ones(rows(x),cols(y)) );
endp;

proc dProc(t,x,y);
  retp( (r - mu + eta*(Vbar-x) ) .* x .* ones(rows(x),cols(y)) );
endp;

proc eProc(t,x,y);
  retp( 0 .* ones(rows(x),cols(y)) );
endp;

proc fProc(t,x,y);
  retp( r .* ones(rows(x),cols(y)) );
endp;

proc gProc(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc tminBound(t,x,y);
  retp( rndu(rows(x),cols(y)) );
endp;

proc xminBound(t,x,y);
  retp( 0.*ones(rows(x),cols(y)) );
endp;

proc xmaxBound(t,x,y);
  retp( (x - I) .* ones(rows(x),cols(y)) );
endp;

proc DxmaxBound(t,x,y);
  retp( ones(rows(x),cols(y)) );
endp;


proc DyminBound(t,x,y);
  retp( 0*ones(rows(x),cols(y)) );
endp;

proc DymaxBound(t,x,y);
  retp( 0*ones(rows(x),cols(y)) );
endp;

__output = 1;

_Hopscotch_SaveLastIter = 1;
_Hopscotch_Elliptic = 1;
_fcmptol = 1e-4;
_Hopscotch_PrintIters = 10;

output file = wp12.out reset;  

V = 1.585|1.68;

sol = {};
x = {};
e = 1;

do until e > 2;

  Vstar = V[e];

  call PDE2D(&aProc,&bProc,&cProc,&dProc,&eProc,&fProc,&gProc,0,
                    &tminBound,&xminBound,&xmaxBound,0,0,
                    0,0,&DyminBound,&DymaxBound);

  xmin = 0.000; xmax = Vstar; Nx = 51;
  ymin = 0.000; ymax = 1; Ny = 5;
  tmin = 0; tmax = 100; Nt = 1001;

  call Hopscotch(tmin,tmax,Nt,xmin,xmax,Nx,ymin,ymax,Ny,"tmp");

  x = x ~ ExtractSolution("tmp","x");
  y = ExtractSolution("tmp","y");
  t = ExtractSolution("tmp","t");

  U = ExtractSolution("tmp","t"|t);

  hx = vread(_Hopscotch_MeshRatios,"hx");

  sol = sol~U[.,3];

  e = e + 1;
endo;

V = seqa(1.0,0.05,51);
graphset;
  _pdate = ""; _pnum = 2; _pltype = 1|3|6;
  xlabel("V");
  ylabel("C(V)");
  ytics(0,0.8,0.1,2);
  xtics(0,2.0,0.2,2);
  _plegstr = "Mean Reversion I\000Mean Reversion II\000V - I";
  _plegctl = {2 5 2.5 4};
  _pline = 1~5~1.585~0~1.585~0.585~1~7~5 |
           1~5~1.680~0~1.680~0.680~1~7~5 ;
  graphprt("-c=1 -cf=pde2d6.eps");
  xy(x[.,2 1]~V,sol[.,2 1]~(V-I));

