new;
library tsm,optmum,pgraph;


/*
================== First example: Time-varying parameters ==================
*/

@<--- data generation --->@

rndseed 456;

Nobs = 1000;

s = seqa(1,1,Nobs);

x = floor(rndu(Nobs,3)*100);

b1 = ones(Nobs,1);                                      /* constant parameter */
b2 = sin(seqa(0,2*pi/Nobs,Nobs)) + rndn(Nobs,1)*0.25;   /* sine parameter     */
b3 = recserar(rndn(Nobs,1),10,0.9);                     /* AR parameter       */
B = b1~b2~b3;

y = sumc(x'.*B') + rndn(Nobs,1);


/* Flexible Least Squares with mu = 1 
** (and imposing constancy for the first parameter)
*/

mu = 100000|1|1;
{Bfls,u,r2_M,r2_D} = FLS(y,x,mu);          /* FLS estimation */

Bols = (invpd(x'x)*x'y )' .*ones(Nobs,1);  /* OLS estimation */

xy1 = b1~Bfls[.,1]~Bols[.,1];
xy2 = b2~Bfls[.,2]~Bols[.,2];
xy3 = b3~Bfls[.,3]~Bols[.,3];


/*
================== Second example: step function ==================
*/

StepValues = 2|2.5|2.25|1.5|2|2.5|3|2.75;

x1 = floor(rndu(Nobs,1)*100); x2 = floor(rndu(Nobs,2)*100);
b1 = {}; b2 = 2|3;
i = 1;
do until i > 8;
  b1 = b1 | StepValues[i]*ones(125,1);
  i = i + 1;
endo;

y = 4 + x1.*b1 + x2*b2 + rndn(Nobs,1);
x = ones(Nobs,1)~x1~x2;

{Bfls,u,r2M,r2D} = FLS(y,x,100000);          /* FLS estimation */

Bols = (invpd(x'x)*x'y )' .*ones(Nobs,1);  /* OLS estimation */

xy4 = b1~Bfls[.,2]~Bols[.,2];

graphset;
  begwind;
  window(2,2,0);

    _pdate = ""; _pnum = 2; _pnumht = 0.20; _paxht = 0.20;
    _pltype = 6|1|3; 
    _plegstr = "true\0FLS\0OLS"; _plegctl = {2 8 3 2};
    xlabel("t");

  setwind(1);
    title("Time-varying parameters\LFirst coefficient");
    xy(s,xy1);

    _plegctl = 0;

  setwind(2);
    title("Time-varying parameters\Lsecond coefficient");
    xy(s,xy2);

  setwind(3);
    title("Time-varying parameters\LThird coefficient");
    xy(s,xy3);

  setwind(4);
    title("step functions");
    xy(s,xy4);

  graphprt("-c=1 -cf=tsm2e.eps");

  endwind;