/*
**  Spectral Kolmogorov-Smirnov test 
**
**  BROCKWELL and DAVIS [1991], Times Series: Theory and Methods,
**  Springer-Verlag, New York, pages 339-342
*/

new;
library tsm,optmum,pgraph;

load data[] = frfdem.asc;

r = packr(log(data./lag1(data))); /* Returns */

y1 = fractional_filter(r,-0.20);    
y2 = fractional_filter(r,0);    
y3 = fractional_filter(r,0.20);    

_fourier = 0;
{lambda,I1} = PDGM(y1);
{lambda,I2} = PDGM(y2);
{lambda,I3} = PDGM(y3);

q = rows(lambda)/2;

C1 = cumsumc(I1[1:q])/sumc(I1[1:q]);
C2 = cumsumc(I2[1:q])/sumc(I2[1:q]);
C3 = cumsumc(I3[1:q])/sumc(I3[1:q]);

x = seqa(1,1,q);

k_alpha = -1.63~1.63;                          /* 1% level */
C_bound = (x-1)/(q-1) + k_alpha*(q-1)^(-0.5);

graphset;
  title("Kolmogorov-Smirnov test using the spectral analysis"\
      "\LTesting for a fractional process");
  _pdate = ""; _pnum = 2;
  _pltype = 6|6|6|1|1;
  _pstype = 8|9|10|1|1;
  _plctrl = 100|100|100|0|0;
  fonts("simplex simgrma");
  xtics(0,pi,pi/4,0); ytics(0,1,0.25,0);
  lab = " 0 \202p\201/4 \202p\201/2 \2013\202p\201/4 \202p\201";
  asclabel(lab,0);
  xlabel("frequency");
  graphprt("-c=1 -cf=tsm4d.eps");
  xy(x/q*pi,C1~C2~C3~C_bound);


